module i2c_control(
	clk,
	rst,
	nReset,
	ena,
	clk_cnt,
	start,
	stop,
	read,
	write,
	ack_in,
	din,
	scl_i,
	sda_i,
	cmd_ack,
	ack_out,
	i2c_busy,
	i2c_al,
	dout,
	scl_o,
	scl_oen,
	sda_o,
	sda_oen);

	//---Ports declearation: generated by Robei---
	input clk;
	input rst;
	input nReset;
	input ena;
	input [15:0] clk_cnt;
	input start;
	input stop;
	input read;
	input write;
	input ack_in;
	input [7:0] din;
	input scl_i;
	input sda_i;
	output cmd_ack;
	output ack_out;
	output i2c_busy;
	output i2c_al;
	output [7:0] dout;
	output scl_o;
	output scl_oen;
	output sda_o;
	output sda_oen;

	wire clk;
	wire rst;
	wire nReset;
	wire ena;
	wire [15:0] clk_cnt;
	wire start;
	wire stop;
	wire read;
	wire write;
	wire ack_in;
	wire [7:0] din;
	wire scl_i;
	wire sda_i;
	reg cmd_ack;
	reg ack_out;
	wire i2c_busy;
	wire i2c_al;
	wire [7:0] dout;
	wire scl_o;
	wire scl_oen;
	wire sda_o;
	wire sda_oen;

	//----Code starts here: integrated by Robei-----
	parameter [4:0] ST_IDLE  = 5'b0_0000;
		parameter [4:0] ST_START = 5'b0_0001;
		parameter [4:0] ST_READ  = 5'b0_0010;
		parameter [4:0] ST_WRITE = 5'b0_0100;
		parameter [4:0] ST_ACK   = 5'b0_1000;
		parameter [4:0] ST_STOP  = 5'b1_0000;
	
		reg  [3:0] core_cmd;
		reg        core_txd;
		wire       core_ack, core_rxd;
	
		reg [7:0] sr;
		reg       shift, ld;
	
		wire       go;
		reg  [2:0] dcnt;
		wire       cnt_done;
	
	
	
	
		assign go = (read | write | stop) & ~cmd_ack;
	
	
		assign dout = sr;
	
		always @(posedge clk or negedge nReset)
		  if (!nReset)
		    sr <= #1 8'h0;
		  else if (rst)
		    sr <= #1 8'h0;
		  else if (ld)
		    sr <= #1 din;
		  else if (shift)
		    sr <= #1 {sr[6:0], core_rxd};
	
		always @(posedge clk or negedge nReset)
		  if (!nReset)
		    dcnt <= #1 3'h0;
		  else if (rst)
		    dcnt <= #1 3'h0;
		  else if (ld)
		    dcnt <= #1 3'h7;
		  else if (shift)
		    dcnt <= #1 dcnt - 3'h1;
	
		assign cnt_done = ~(|dcnt);
	
		
		reg [4:0] c_state; 
	
		always @(posedge clk or negedge nReset)
		  if (!nReset)
		    begin
		        core_cmd <= #1 4'b0000;
		        core_txd <= #1 1'b0;
		        shift    <= #1 1'b0;
		        ld       <= #1 1'b0;
		        cmd_ack  <= #1 1'b0;
		        c_state  <= #1 ST_IDLE;
		        ack_out  <= #1 1'b0;
		    end
		  else if (rst | i2c_al)
		   begin
		       core_cmd <= #1 4'b0000;
		       core_txd <= #1 1'b0;
		       shift    <= #1 1'b0;
		       ld       <= #1 1'b0;
		       cmd_ack  <= #1 1'b0;
		       c_state  <= #1 ST_IDLE;
		       ack_out  <= #1 1'b0;
		   end
		else
		  begin
		      // initially reset all signals
		      core_txd <= #1 sr[7];
		      shift    <= #1 1'b0;
		      ld       <= #1 1'b0;
		      cmd_ack  <= #1 1'b0;
	
		      case (c_state) 
		        ST_IDLE:
		          if (go)
		            begin
		                if (start)
		                  begin
		                      c_state  <= #1 ST_START;
		                      core_cmd <= #1 4'b0001;
		                  end
		                else if (read)
		                  begin
		                      c_state  <= #1 ST_READ;
		                      core_cmd <= #1 4'b1000;
		                  end
		                else if (write)
		                  begin
		                      c_state  <= #1 ST_WRITE;
		                      core_cmd <= #1 4'b0100;
		                  end
		                else // stop
		                  begin
		                      c_state  <= #1 ST_STOP;
		                      core_cmd <= #1 4'b0010;
		                  end
	
		                ld <= #1 1'b1;
		            end
	
		        ST_START:
		          if (core_ack)
		            begin
		                if (read)
		                  begin
		                      c_state  <= #1 ST_READ;
		                      core_cmd <= #1 4'b1000;
		                  end
		                else
		                  begin
		                      c_state  <= #1 ST_WRITE;
		                      core_cmd <= #1 4'b0100;
		                  end
	
		                ld <= #1 1'b1;
		            end
	
		        ST_WRITE:
		          if (core_ack)
		            if (cnt_done)
		              begin
		                  c_state  <= #1 ST_ACK;
		                  core_cmd <= #1 4'b1000;
		              end
		            else
		              begin
		                  c_state  <= #1 ST_WRITE;      
		                  core_cmd <= #1 4'b0100;
		                  shift    <= #1 1'b1;
		              end
	
		        ST_READ:
		          if (core_ack)
		            begin
		                if (cnt_done)
		                  begin
		                      c_state  <= #1 ST_ACK;
		                      core_cmd <= #1 4'b0100;
		                  end
		                else
		                  begin
		                      c_state  <= #1 ST_READ;      
		                      core_cmd <= #1 4'b1000; 
		                  end
	
		                shift    <= #1 1'b1;
		                core_txd <= #1 ack_in;
		            end
	
		        ST_ACK:
		          if (core_ack)
		            begin
		               if (stop)
		                 begin
		                     c_state  <= #1 ST_STOP;
		                     core_cmd <= #1 4'b0010;
		                 end
		               else
		                 begin
		                     c_state  <= #1 ST_IDLE;
		                     core_cmd <= #1 4'b0000;
	
		                
		                     cmd_ack  <= #1 1'b1;
		                 end
	
		               
		                 ack_out <= #1 core_rxd;
	
		                 core_txd <= #1 1'b1;
		             end
		           else
		             core_txd <= #1 ack_in;
	
		        ST_STOP:
		          if (core_ack)
		            begin
		                c_state  <= #1 ST_IDLE;
		                core_cmd <= #1 4'b0000;
	
		              
		                cmd_ack  <= #1 1'b1;
		            end
	
		      endcase
		  end
	
	
	
	//---Module instantiation---
	i2c_master_control i2c_master_control1(
		.clk(clk),
		.rst(rst),
		.nReset(nReset),
		.ena(ena),
		.clk_cnt(clk_cnt),
		.scl_i(scl_i),
		.sda_i(sda_i),
		.busy(i2c_busy),
		.al(i2c_al),
		.scl_o(scl_o),
		.scl_oen(scl_oen),
		.sda_o(sda_o),
		.sda_oen(sda_oen));

endmodule    //i2c_control

