module i2c_master_top(
	rst,
	clk,
	clk_div_cnt,
	scl_pad_i,
	sda_pad_i,
	i2c_addr_2byte,
	i2c_read_req,
	i2c_write_req,
	i2c_slave_dev_addr,
	i2c_slave_reg_addr,
	i2c_write_data,
	scl_pad_o,
	scl_padoen_o,
	sda_pad_o,
	sda_padoen_o,
	i2c_read_req_ack,
	i2c_write_req_ack,
	i2c_read_data,
	error,
	p19,
	p20,
	irxack,
	rxr,
	i2c_busy,
	i2c_al,
	done);

	//---Ports declearation: generated by Robei---
	input rst;
	input clk;
	input [15:0] clk_div_cnt;
	input scl_pad_i;
	input sda_pad_i;
	input i2c_addr_2byte;
	input i2c_read_req;
	input i2c_write_req;
	input [7:0] i2c_slave_dev_addr;
	input [15:0] i2c_slave_reg_addr;
	input [7:0] i2c_write_data;
	output scl_pad_o;
	output scl_padoen_o;
	output sda_pad_o;
	output sda_padoen_o;
	output i2c_read_req_ack;
	output i2c_write_req_ack;
	output [7:0] i2c_read_data;
	output error;
	input p19;
	input p20;
	output irxack;
	output [7:0] rxr;
	output i2c_busy;
	output i2c_al;
	output done;

	wire rst;
	wire clk;
	wire [15:0] clk_div_cnt;
	wire scl_pad_i;
	wire sda_pad_i;
	wire i2c_addr_2byte;
	wire i2c_read_req;
	wire i2c_write_req;
	wire [7:0] i2c_slave_dev_addr;
	wire [15:0] i2c_slave_reg_addr;
	wire [7:0] i2c_write_data;
	wire scl_pad_o;
	wire scl_padoen_o;
	wire sda_pad_o;
	wire sda_padoen_o;
	wire i2c_read_req_ack;
	wire i2c_write_req_ack;
	reg [7:0] i2c_read_data;
	reg error;
	wire p19;
	wire p20;
	wire irxack;
	wire [7:0] rxr;
	wire i2c_busy;
	wire i2c_al;
	wire done;

	//----Code starts here: integrated by Robei-----
	parameter S_IDLE             =  0;             
	parameter S_WR_DEV_ADDR      =  1;           
	parameter S_WR_REG_ADDR      =  2;            
	parameter S_WR_DATA          =  3;           
	parameter S_WR_ACK           =  4;             
	parameter S_WR_ERR_NACK      =  5;            
	parameter S_RD_DEV_ADDR0     =  6;            
	parameter S_RD_REG_ADDR      =  7;            
	parameter S_RD_DEV_ADDR1     =  8;          
	parameter S_RD_DATA          =  9;             
	parameter S_RD_STOP          = 10;  
	parameter S_WR_STOP          = 11; 
	parameter S_WAIT             = 12; 
	parameter S_WR_REG_ADDR1     = 13; 
	parameter S_RD_REG_ADDR1     = 14; 
	parameter S_RD_ACK           = 15; 
	reg start;
	reg stop;
	reg read;
	reg write;
	reg ack_in;
	reg[7:0] txr;
	
	reg[3:0] state, next_state;
	assign i2c_read_req_ack = (state == S_RD_ACK);
	assign i2c_write_req_ack = (state == S_WR_ACK);
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			state <= S_IDLE;
		else
			state <= next_state;    
	end
	always@(*)
	begin
		case(state)
			S_IDLE:
		
				if(i2c_write_req)
					next_state <= S_WR_DEV_ADDR;
				else if(i2c_read_req)
					next_state <= S_RD_DEV_ADDR0;
				else
					next_state <= S_IDLE;
		
			S_WR_DEV_ADDR:
				if(done && irxack)
					next_state <= S_WR_ERR_NACK;
				else if(done)
					next_state <= S_WR_REG_ADDR;
				else
					next_state <= S_WR_DEV_ADDR;
		
			S_WR_REG_ADDR:
				if(done)
					
					next_state <= i2c_addr_2byte ? S_WR_REG_ADDR1 : S_WR_DATA;
				else
					next_state <= S_WR_REG_ADDR;
			S_WR_REG_ADDR1:
				if(done)
					next_state <= S_WR_DATA;
				else
					next_state <= S_WR_REG_ADDR1; 
				
			S_WR_DATA:
				if(done)
					next_state <= S_WR_STOP;
				else
					next_state <= S_WR_DATA;
			S_WR_ERR_NACK:
				next_state <= S_WR_STOP;
			S_RD_ACK,S_WR_ACK:
				next_state <= S_WAIT;
			S_WAIT:
				next_state <= S_IDLE;
			S_RD_DEV_ADDR0:
				if(done && irxack)
					next_state <= S_WR_ERR_NACK;
				else if(done)
					next_state <= S_RD_REG_ADDR;
				else
					next_state <= S_RD_DEV_ADDR0;
			S_RD_REG_ADDR:
				if(done)
					next_state <= i2c_addr_2byte ? S_RD_REG_ADDR1 : S_RD_DEV_ADDR1;
				else
					next_state <= S_RD_REG_ADDR;
			S_RD_REG_ADDR1:
				if(done)
					next_state <= S_RD_DEV_ADDR1;
				else
					next_state <= S_RD_REG_ADDR1;               
			S_RD_DEV_ADDR1:
				if(done)
					next_state <= S_RD_DATA;
				else
					next_state <= S_RD_DEV_ADDR1;   
			S_RD_DATA:
				if(done)
					next_state <= S_RD_STOP;
				else
					next_state <= S_RD_DATA;
			S_RD_STOP:
				if(done)
					next_state <= S_RD_ACK;
				else
					next_state <= S_RD_STOP;
			S_WR_STOP:
				if(done)
					next_state <= S_WR_ACK;
				else
					next_state <= S_WR_STOP;                
			default:
				next_state <= S_IDLE;
		endcase
	end
	
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			error <= 1'b0;
		else if(state == S_IDLE)
			error <= 1'b0;
		else if(state == S_WR_ERR_NACK)
			error <= 1'b1;
	end
	
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			start <= 1'b0;
		else if(done)
			start <= 1'b0;
		else if(state == S_WR_DEV_ADDR || state == S_RD_DEV_ADDR0 || state == S_RD_DEV_ADDR1)
			start <= 1'b1;
	end
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			stop <= 1'b0;
		else if(done)
			stop <= 1'b0;
		else if(state == S_WR_STOP || state == S_RD_STOP)
			stop <= 1'b1;
	end
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			ack_in <= 1'b0;
		else 
			ack_in <= 1'b1;
	end
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			write <= 1'b0;
		else if(done)
			write <= 1'b0;
		else if(state == S_WR_DEV_ADDR || state == S_WR_REG_ADDR || state == S_WR_REG_ADDR1|| state == S_WR_DATA || state == S_RD_DEV_ADDR0 || state == S_RD_DEV_ADDR1 || state == S_RD_REG_ADDR || state == S_RD_REG_ADDR1)
			write <= 1'b1;
	end
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			read <= 1'b0;
		else if(done)
			read <= 1'b0;
		else if(state == S_RD_DATA)
			read <= 1'b1;
	end
	
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			i2c_read_data <= 8'h00;
		else if(state == S_RD_DATA && done)
			i2c_read_data <= rxr;
	end
	
	always@(posedge clk or posedge rst)
	begin
		if(rst)
			txr <= 8'd0;
		else 
			case(state)
				S_WR_DEV_ADDR,S_RD_DEV_ADDR0:
					txr <= {i2c_slave_dev_addr[7:1],1'b0};
				S_RD_DEV_ADDR1:
					txr <= {i2c_slave_dev_addr[7:1],1'b1};
				S_WR_REG_ADDR,S_RD_REG_ADDR:
					txr <= (i2c_addr_2byte == 1'b1) ? i2c_slave_reg_addr[15:8] : i2c_slave_reg_addr[7:0];
				S_WR_REG_ADDR1,S_RD_REG_ADDR1:
					txr <= i2c_slave_reg_addr[7:0];             
				S_WR_DATA:
					txr <= i2c_write_data;
				default:
					txr <= 8'hff;
			endcase
	end
	i2c_control byte_controller (
		.clk      ( clk          ),
		.rst      ( rst          ),
		.nReset   ( 1'b1         ),
		.ena      ( 1'b1         ),
		.clk_cnt  ( clk_div_cnt  ),
		.start    ( start        ),
		.stop     ( stop         ),
		.read     ( read         ),
		.write    ( write        ),
		.ack_in   ( ack_in       ),
		.din      ( txr          ),
		.cmd_ack  ( done         ),
		.ack_out  ( irxack       ),
		.dout     ( rxr          ),
		.i2c_busy ( i2c_busy     ),
		.i2c_al   ( i2c_al       ),
		.scl_i    ( scl_pad_i    ),
		.scl_o    ( scl_pad_o    ),
		.scl_oen  ( scl_padoen_o ),
		.sda_i    ( sda_pad_i    ),
		.sda_o    ( sda_pad_o    ),
		.sda_oen  ( sda_padoen_o )
	);
	
	
	
	//---Module instantiation---
	i2c_control i2c_control1(
		.clk(clk),
		.rst(rst),
		.nReset(p19),
		.ena(p20),
		.clk_cnt(clk_div_cnt),
		.scl_i(scl_pad_i),
		.sda_i(sda_pad_i),
		.cmd_ack(done),
		.ack_out(irxack),
		.i2c_busy(i2c_busy),
		.i2c_al(i2c_al),
		.dout(rxr),
		.scl_o(scl_pad_o),
		.scl_oen(scl_padoen_o),
		.sda_o(sda_pad_o),
		.sda_oen(sda_padoen_o));

endmodule    //i2c_master_top

