module IIC_Driver(
	sys_clk,
	rst_n,
	IICWriteReq,
	IICSCL,
	IICWriteDone,
	IICReadDone,
	IICReadData,
	IICSDA,
	IICWriteData,
	IICReadReq,
	IICSlave);

	//---Ports declearation: generated by Robei---
	input sys_clk;
	input rst_n;
	input IICWriteReq;
	output IICSCL;
	output IICWriteDone;
	output IICReadDone;
	output [7:0] IICReadData;
	inout IICSDA;
	input [7:0] IICWriteData;
	input IICReadReq;
	input [15:0] IICSlave;

	wire sys_clk;
	wire rst_n;
	wire IICWriteReq;
	wire IICSCL;
	wire IICWriteDone;
	wire IICReadDone;
	wire [7:0] IICReadData;
	wire IICSDA;
	wire [7:0] IICWriteData;
	wire IICReadReq;
	wire [15:0] IICSlave;

	//----Code starts here: integrated by Robei-----
	
	
	localparam IIC_IDLE       =   6'b000_001; 
	localparam IIC_START      =   6'b000_010;  
	localparam IIC_WRDATA     =   6'b000_100; 
	localparam IIC_RDDATA     =   6'b001_000;  
	localparam IIC_ACK        =   6'b010_000; 
	localparam IIC_STOP       =   6'b100_000;
	
	
	
	localparam  IIC_Pre       =   'd100;   
	reg[5:0]   state , next_state;
	reg[21:0]  IICCnt;             
	reg[3:0]   IICBitCnt;           
	reg[1:0]   IICACKStopCnt;      
	reg[2:0]   IICSendBytes;       
	
	
	reg[15:0]   IICSlaveReg;           
	reg[7:0]   IICReadDataReg;     
	
	
	reg 	IICWriteReqReg;
	
	reg   iictx;     
	reg   iicCLK;   
	
	assign IICSDA = (state == IIC_RDDATA || (state == IIC_ACK)) ? 1'bz : iictx; 
	assign IICSCL =  iicCLK;
	
	assign IICReadData = IICReadDataReg;
	
	assign IICReadDone  = (state != next_state && state == IIC_STOP) ? 1'b1 : 1'b0;
	assign IICWriteDone = (state != next_state && state == IIC_STOP) ? 1'b1 : 1'b0;
	
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
		if(rst_n == 1'b0)
			IICWriteReqReg <= 1'b0;
		else if(IICWriteDone == 1'b1)
			IICWriteReqReg <= 1'b0;
		else if(IICWriteReq == 1'b1)
		   IICWriteReqReg <= 1'b1;
		else
			IICWriteReqReg <= IICWriteReqReg;
	
	end
	
	always @(posedge sys_clk or negedge rst_n) 
	begin
	    if(rst_n == 1'b0)
	        state <= IIC_IDLE;
	    else
	        state <= next_state;   
	end
	
	/*状态机*/
	always @(*)
	begin
	    case(state)
	    IIC_IDLE: 
	        if(IICWriteReq == 1'b1 || IICReadReq == 1'b1)
	            next_state <= IIC_START;
	        else
	            next_state <= IIC_IDLE;
	    IIC_START:
	        if(IICCnt == (IIC_Pre * 'd2))
	            next_state <= IIC_WRDATA;
	        else
	            next_state <= IIC_START;
	    IIC_WRDATA:
	        if(IICBitCnt == 'd8 /*&& IICCnt == IIC_Pre /4 */&& iicCLK == 1'b0)
	            next_state <= IIC_ACK;
	        else
	            next_state <= IIC_WRDATA;
	    IIC_RDDATA:
	        if(IICBitCnt == 'd8 && IICCnt == IIC_Pre /4 && iicCLK == 1'b0)
	            next_state <= IIC_ACK;
	        else
	            next_state <= IIC_RDDATA;
	    IIC_ACK:
	        if(IICACKStopCnt == 'd1 /*&& IICCnt == IIC_Pre /4 */&& iicCLK == 1'b0)
	            if(IICSendBytes == 'd2) 
	                if(/*IICWriteReq*/IICWriteReqReg == 1'b1)             
	                    next_state <= IIC_STOP;
	                else 
	                    next_state <= IIC_RDDATA;
	            else if(IICSendBytes == 'd2 && IICReadReq == 1'b1)
	                next_state <= IIC_START;
	            else if(IICSendBytes == 'd4)
	                next_state <= IIC_STOP;
	            else
	                next_state <= IIC_WRDATA;
	        else
	            next_state <= IIC_ACK;
	    IIC_STOP:
	        if(IICACKStopCnt == 'd1 && IICCnt == IIC_Pre/4 && iicCLK == 1'b1)
	            next_state <= IIC_IDLE;
	        else
	            next_state <= IIC_STOP;
	    default:  next_state <= IIC_IDLE;
	    endcase
	end
	
	
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICSendBytes <= 'd0;
	    else if(state == IIC_IDLE)
	        IICSendBytes <= 'd0;
	    else if(state == IIC_ACK)
	        if(next_state != state)
	            IICSendBytes <= IICSendBytes + 1'b1;
	        else
	            IICSendBytes <= IICSendBytes;
	    else
	        IICSendBytes <= IICSendBytes;
	
	end
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICCnt <= 'd0;
	    else if(IICCnt == IIC_Pre && state != IIC_START)
	        IICCnt <= 'd0;
	    else if(IICCnt == (IIC_Pre *'d2) && state != IIC_START)
	        IICCnt <= 'd0;
	    else if(state != next_state)
	        IICCnt <= 'd0;
	    else if(state == IIC_START)
	        IICCnt <= IICCnt + 1'b1;
	    else if(state == IIC_WRDATA )
	        IICCnt <= IICCnt + 1'b1;
	    else if(state == IIC_RDDATA)
	        IICCnt <= IICCnt + 1'b1;
	    else if(state == IIC_ACK)
	        IICCnt <= IICCnt + 1'b1;
	    else if(state == IIC_STOP)
	        IICCnt <= IICCnt + 1'b1;
	end
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICBitCnt <= 'd0;
	    else if(state == IIC_IDLE || state == IIC_ACK)
	        IICBitCnt <= 'd0;
	    else if(state == IIC_WRDATA && IICCnt == (IIC_Pre /2))
	        if(iicCLK == 1'b1)
	            IICBitCnt <= IICBitCnt + 1'b1;
	        else
	            IICBitCnt <= IICBitCnt;
	    else if(state == IIC_RDDATA && IICCnt == (IIC_Pre /2))
	        if(iicCLK == 1'b1)
	            IICBitCnt <= IICBitCnt + 1'b1;
	        else
	            IICBitCnt <= IICBitCnt;
	    else
	        IICBitCnt <= IICBitCnt;
	end
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICACKStopCnt <= 'd0;
	    else if(state != next_state)
	        IICACKStopCnt <= 'd0;
	    else if((state == IIC_ACK || state == IIC_STOP) && IICCnt == (IIC_Pre /2))
	        if(iicCLK == 1'b1)
	            IICACKStopCnt <= IICACKStopCnt + 1'b1;
	        else
	            IICACKStopCnt <= IICACKStopCnt;
	    else if(state == IIC_ACK || state == IIC_STOP)
	        IICACKStopCnt <= IICACKStopCnt;
	    else
	        IICACKStopCnt <= 'd0;
	end
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICSlaveReg <= 'd0;
	    else if((IICWriteReq == 1'b1 || IICReadReq == 1'b1) && state == IIC_IDLE)  
	        IICSlaveReg <= IICSlave;
	    else if(state == IIC_ACK && state != next_state)   
	        if(IICSendBytes == 'd2)
	            IICSlaveReg <= {IICSlaveReg[7:0],IICSlaveReg[15:8]} + 1'b1;
	        else
	            IICSlaveReg <= {IICSlaveReg[7:0],IICSlaveReg[15:8]};
	    else
	        IICSlaveReg <= IICSlaveReg;
	end
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        iicCLK <= 1'b1;
	    else if(state == IIC_START && IICCnt == (IIC_Pre*2))      /*开始结束时，拉低时钟线*/
	        iicCLK <= 1'b0;
	    else if(state == IIC_START && IICCnt > IIC_Pre)
	        iicCLK <= 1'b1;
	    else if(state == IIC_START && IICCnt == IIC_Pre)
	        iicCLK <= iicCLK;
	    else if(state == IIC_WRDATA && IICCnt == IIC_Pre)     /*发送数据，依次取反时钟/
	        iicCLK <= ~iicCLK;
	    else if(state == IIC_RDDATA && IICCnt == IIC_Pre)     /*接收数据，依次取反时钟线*/
	        iicCLK <= ~iicCLK;
	    else if(state == IIC_ACK && IICCnt == IIC_Pre)
	        iicCLK <= ~iicCLK;
	    else if(state == IIC_STOP && IICCnt == IIC_Pre)  /*在停止态时，IICCnt时，直接拉高iicCLK*/
	        iicCLK <= 1'b1;
	end
	
	
	
	
	always@(posedge sys_clk or negedge rst_n)
	begin
	    if(rst_n == 1'b0)
	        IICReadDataReg <= 'd0;
	    else if(state == IIC_RDDATA && IICCnt == IIC_Pre / 2)
	        if(iicCLK == 1'b1)
	            IICReadDataReg <= {IICReadDataReg[6:0],IICSDA};
	        else
	            IICReadDataReg <= IICReadDataReg;
	    else
	        IICReadDataReg <= IICReadDataReg;
	end
	
	
	
	
	
	
endmodule    //IIC_Driver

